package it.eng.spago.cms.exec;

import it.eng.spago.cms.CmsNode;
import it.eng.spago.cms.IExecutionService;
import it.eng.spago.cms.constants.Constants;
import it.eng.spago.cms.exceptions.OperationExecutionException;
import it.eng.spago.cms.init.CMSManager;
import it.eng.spago.cms.operations.DeleteOperation;
import it.eng.spago.cms.operations.GetOperation;
import it.eng.spago.cms.operations.RestoreOperation;
import it.eng.spago.cms.operations.SearchOperation;
import it.eng.spago.cms.operations.SetOperation;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;

import java.util.List;

/**
 * Implementation of the IExecutionService interface: implements all methods to perform 
 * different operations towards the jcr repository
 */
public class InProcessOperationExecutor implements IExecutionService {

	/**
	 * Execs a Get operation using a default user credentials
	 * @param getOp Configuration of the get operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public CmsNode doGetOperation(GetOperation getOp) throws OperationExecutionException {
		CmsNode node = null;
		CMSConnection connection = getConnection();
		GetObjectImpl getObjImpl = new GetObjectImpl();
		node = getObjImpl.execute(getOp.getOperationDescriptor(), connection);
		closeConnection(connection);
		return node; 
	}

	/**
	 * Execs a get operation using a specific user name and password
	 * @param getOp Configuration of the get operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public CmsNode doGetOperation(GetOperation getOp, String name, String password) throws OperationExecutionException {
		CmsNode node = null;
		CMSConnection connection = getConnection();
		GetObjectImpl getObjImpl = new GetObjectImpl();
		node = getObjImpl.execute(getOp.getOperationDescriptor(), connection);
		closeConnection(connection);
		return node; 
	}
	
	/**
	 * Execs a Set operation using a default user credentials
	 * @param setOp Configuration of the set operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doSetOperation(SetOperation setOp) throws OperationExecutionException {
		CMSConnection connection = getConnection();
		SetObjectImpl setObjImpl = new SetObjectImpl();
		setObjImpl.execute(setOp.getOperationDescriptor(), connection);
		closeConnection(connection);
	}

	/**
	 * Execs a Set operation using a specific user name and password
	 * @param setOp Configuration of the set operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doSetOperation(SetOperation setOp, String name, String password) throws OperationExecutionException {
		CMSConnection connection = getConnection();
		SetObjectImpl setObjImpl = new SetObjectImpl();
		setObjImpl.execute(setOp.getOperationDescriptor(), connection);
		closeConnection(connection);
	}
	
	/**
	 * Execs a delete operation using a default user credentials
	 * @param delOp Configuration of the delete operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doDeleteOperation(DeleteOperation delOp) throws OperationExecutionException {
		CMSConnection connection = getConnection();
		DeleteObjectImpl delObjImpl = new DeleteObjectImpl();
		delObjImpl.execute(delOp.getOperationDescriptor(), connection);
		closeConnection(connection);
	}
	/**
	 * Execs a delete operation using a specific user name and password
	 * @param delOp Configuration of the delete operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doDeleteOperation(DeleteOperation delOp, String name, String password) throws OperationExecutionException {
		CMSConnection connection = getConnection();
		DeleteObjectImpl delObjImpl = new DeleteObjectImpl();
		delObjImpl.execute(delOp.getOperationDescriptor(), connection);
		closeConnection(connection);
	}
	
	/**
	 * Execs a restore operation using a default user credentials
	 * @param restOp Configuration of the restore operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doRestoreOperation(RestoreOperation resOp) throws OperationExecutionException {
		CMSConnection connection = getConnection();
		RestoreObjectImpl resObjImpl = new RestoreObjectImpl();
		resObjImpl.execute(resOp.getOperationDescriptor(), connection);
		closeConnection(connection);
	}
	/**
	 * Execs a restore operation using a specific user name and password
	 * @param restOp Configuration of the restore operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doRestoreOperation(RestoreOperation resOp, String name, String password) throws OperationExecutionException {
		CMSConnection connection = getConnection();
		RestoreObjectImpl resObjImpl = new RestoreObjectImpl();
		resObjImpl.execute(resOp.getOperationDescriptor(), connection);
		closeConnection(connection);
	}
	
	/**
	 * Execs a Search operation using a default user credentials
	 * @param searchOp Configuration of the search operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public List doSearchOperation(SearchOperation searchOp) throws OperationExecutionException {
		List nodes = null;
		CMSConnection connection = getConnection();
		SearchObjectImpl searchObjImpl = new SearchObjectImpl();
		nodes = searchObjImpl.execute(searchOp.getOperationDescriptor(), connection);
		closeConnection(connection);
		return nodes; 
	}
	/**
	 * Execs a search operation using a specific user name and password
	 * @param searchOp Configuration of the search operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public List doSearchOperation(SearchOperation searchOp, String name, String password) throws OperationExecutionException {
		List nodes = null;
		CMSConnection connection = getConnection();
		SearchObjectImpl searchObjImpl = new SearchObjectImpl();
		nodes = searchObjImpl.execute(searchOp.getOperationDescriptor(), connection);
		closeConnection(connection);
		return nodes; 
	}
	
	
	/**
	 * Gets a connection to a jsr 170 repository
	 * @return CMSConnection to the jsr repository
	 */
	private CMSConnection getConnection() {
		CMSConnection connection = null;
	    try {
	    	connection = CMSManager.getInstance().getConnection();
	    } catch (EMFInternalError e1) {
	    	TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.MAJOR,
	    						"DeleteObjectImpl::execute: cannot get a connection");				
	    	return null;
		}
	    return connection;
	}
	
	
	
		
	/**
	 * Closes a connection
	 * @param conn, connection to close
	 */
	private void closeConnection(CMSConnection conn) {
		if( conn != null && !conn.isClose() ) {
			conn.close();
		}
	}
	
	
	
}
